#include <assert.h>

#include "shmac/shmac.h"

#include "sched/sched.h"
#include "core.h"
#include "worker.h"
#include "timer.h"

#include "boot.h"

static void start_cores()
{
    extern void (*shmac_pthreads_init)(void); // crt0.s
    shmac_pthreads_init = &pthreads_work_loop;

    *SYS_READY = 1;
}

static void schedule_me()
{
    // create idle task and setup initial state
    pthreads_work_loop();

    // make me a task!
    swi_trigger(SWI_PTHREAD_MAIN_THREAD);

    // now start the timer interrupts
    start_timer();
}

static void wait_for_cores()
{
    volatile enum core_state core_state;
    for (int i = 0; i < *SYS_CPU_COUNT; i++) {
        do {
            core_state = get_core_state(i);
        } while (core_state == NOT_READY);
    }
}

void boot_pthreads()
{
    assert(!boot_pthreads_called());

    sched_init();

    start_cores();
    schedule_me();
    wait_for_cores();
}

int boot_pthreads_called()
{
    return sched_inited();
}
