#ifndef _SHMAC_PTHREAD_CORE_H_
#define _SHMAC_PTHREAD_CORE_H_

/* This file contains the actual core_t object that encapsulates a CPU's state
 * in a scheduling context. It mostly contains various priority queues, linked lists
 * and buffers of various tasks in various states (RUNNING, BLOCKED, etc..). 

  * Functions that relates to a core's state without specifically being related
  * to queueing is found here. That means: initialization, state and locking.
 */

#include "shmac/mutex.h"
#include "sched/sched.h"

// initializes the necessary data structures. This should be called before any other
// functions defined here. In this implementation, it is called from the first call to
// pthread_create. core_inited() tests whether the given core is initialized.
void	core_init(unsigned int core_num, int (*comparators[])(void*,void*));
int		core_inited(unsigned int core_num);

// Returns a pointer to the core_t object belonging to core_num.
core_t *core_get(unsigned int core_num);

// These two functions keep track of how many times a given CPU has been interrupted
// to context switch.
unsigned int core_epoch_count();
void         core_tick();

// gets a core's state. possibly values defined in sched/enums.h
enum core_state get_core_state(unsigned int core_num);
enum core_state get_my_state();

// set a core's state to the specified value.
void set_core_state(unsigned int core_num, enum core_state state);
void set_my_state(enum core_state state);

// Used to lock/unlock the specified core in accordance with the specified
// lock mode. That is either LOCK (will block until it gets a lock), TRY_LOCK
// (returns 0 if it can't get the lock) and NONE, which is a no-op.
int		core_lock(unsigned int core_num, enum lock_mode lm);
void	core_unlock(unsigned int core_num, int lock);

#endif
