#include <stdlib.h>
#include <errno.h>

#include "data/ring_buffer.h"

#include "core.h"
#include "core_mq.h"

#define ID		(*TILEREG_CPUID)

static core_msg_t msg_buf[MAX_CPUS][RING_DEFAULT_BUFFER_SIZE];
static int msg_buf_idx[MAX_CPUS];

void core_mq_push(unsigned int core_num, core_msg_t message)
{
	int my_id = ID;
	int idx = msg_buf_idx[my_id]++;

	if (msg_buf_idx[my_id] == RING_DEFAULT_BUFFER_SIZE) {
		msg_buf_idx[my_id] = 0;
	}

	msg_buf[my_id][idx] = message;
	core_t *core = core_get(core_num);

	if (!ring_buffer_try_push(&(core->queued_msgs[my_id]),
			(void*)&msg_buf[my_id][idx])) {
		return;
	}

	core->has_queued_msgs = 1;
}

int core_mq_should_process()
{
	// NOTE: not idempotent.

	core_t *me = core_get(ID);

	int any = me->has_queued_msgs;
	me->has_queued_msgs = 0;

	return any;
}
int core_mq_pop(core_msg_t *dst)
{
	core_t *me = core_get(ID);
	ring_buffer_t *bufs = (ring_buffer_t*)me->queued_msgs;
	core_msg_t *msg = NULL;

	for (int c = 0; c < MAX_CPUS; c++) {
		if (ring_buffer_try_pop(&bufs[c], (void*)&msg)) {
			*dst = *msg;
			return 1;
		}
	}

	return 0;
}
