#ifndef _SHMAC_PTHREAD_LINKED_LIST_H_
#define _SHMAC_PTHREAD_LINKED_LIST_H_

/*
 * A doubly-linked list implementation.
 */

typedef struct t_next {
    void 			*value;

    struct t_next 	*prev;
    struct t_next 	*next;
} linkedlist_item_t;

typedef struct linked_list {
    linkedlist_item_t 	*head;
    linkedlist_item_t 	*tail;

    unsigned int 		 count;
} linkedlist_t;

// initializes the linked list. should be called before use.
int linked_list_init(linkedlist_t *ll);

// returns pointer to current list head. same as ll->head
linkedlist_item_t *linked_list_head(linkedlist_t *ll);

// returns pointer to current list tail. same as ll->tail
linkedlist_item_t *linked_list_tail(linkedlist_t *ll);

// removes and returns an item from the head of the list.
linkedlist_item_t *linked_list_pop(linkedlist_t *ll);

// appends an item to the end of the list.
int  linked_list_push(linkedlist_t *ll, void *value);

// appends an item to the end of the list -- user-supplied item
int linked_list_push_item(linkedlist_t *ll, linkedlist_item_t *item);

// removes the specified item from the linked list and reconnects
// its predecessor and successor nodes, if they exist.
int  linked_list_remove(linkedlist_t *ll, linkedlist_item_t *ll_item);

// return size of the linked list in O(1). that information is stored 
// separately, so it does not require a list traversal.
unsigned int linked_list_size(linkedlist_t *ll);

#endif