#include <stdlib.h>
#include "ring_buffer.h"

#define ring_buffer_ok(rb) (rb->buffer_size > 0)

void ring_buffer_init(ring_buffer_t *rb)
{
	if (rb->buffer_size == 0) {
		rb->buffer_size = RING_DEFAULT_BUFFER_SIZE;
	}

	void *ptr = malloc(rb->buffer_size * sizeof(void*));
	if (ptr == NULL) {
		rb->buffer_size = 0;
		return;
	}

	rb->data = ptr;
	rb->head = 0;
	rb->tail = 0;
}

void *ring_buffer_pop(ring_buffer_t *rb)
{
	if (!ring_buffer_ok(rb)) {
		return NULL;
	}

	while (rb->head == rb->tail);

	void *item = rb->data[rb->head++];

	if (rb->head == rb->buffer_size) {
		rb->head = 0;
	}

	return item;
}

int ring_buffer_try_pop(ring_buffer_t *rb, void **dst)
{
	if (!ring_buffer_ok(rb)) {
		return 0;
	}

	if (rb->head == rb->tail) {
		return 0;
	}

	void *item = rb->data[rb->head++];
	*dst = item;

	if (rb->head == rb->buffer_size) {
		rb->head = 0;
	}

	return 1;
}

void ring_buffer_push(ring_buffer_t *rb, void *item)
{
	if (!ring_buffer_ok(rb)) {
		return;
	}

	unsigned int next_tail = (rb->tail == rb->buffer_size - 1)
		? 0 : rb->tail + 1;

	while (next_tail == rb->tail);

	rb->data[rb->tail] = item;
	rb->tail = next_tail;
}

int ring_buffer_try_push(ring_buffer_t *rb, void *item)
{
	if (!ring_buffer_ok(rb)) {
		return 0;
	}

	unsigned int next_tail = (rb->tail == rb->buffer_size - 1)
		? 0 : rb->tail + 1;

	if (next_tail == rb->tail) {
		return 0;
	}

	rb->data[rb->tail] = item;
	rb->tail = next_tail;

	return 1;
}

unsigned int ring_buffer_size(ring_buffer_t *rb)
{
	unsigned int h = rb->head,
				 t = rb->tail;

	return (h <= t) ? (t - h) : (t - h + rb->buffer_size);
}
