#ifndef _SHMAC_PTHREAD_RING_BUFFER_H_
#define _SHMAC_PTHREAD_RING_BUFFER_H_

/*
 * A ring-buffer implementation with both blocking and non-blocking
 * semantics.
 */

// if a size of zero is passed to ring_buffer_init, this size
// is used instead.
#define RING_DEFAULT_BUFFER_SIZE 16

typedef struct {
	void	   **data;
	size_t		 buffer_size;
	unsigned int head;
	unsigned int tail;
} ring_buffer_t;

// initialize the ring buffer structure. Must be called before use.
void ring_buffer_init(ring_buffer_t *rb);

// removes the head of the ring buffer. blocks if the ring buffer is empty.
void *ring_buffer_pop(ring_buffer_t *rb);

// removes the head of the ring buffer and returns 1 
// if not empty. otherwise returns 0.
int   ring_buffer_try_pop(ring_buffer_t *rb, void **dst);

// appends item to the end of the ring buffer. blocks if the ring 
// buffer is currently full.
void ring_buffer_push(ring_buffer_t *rb, void *item);

// appends item to the end of the ring buffer if there is space. 
// returns 1 if successful and 0 if the buffer is full.
int  ring_buffer_try_push(ring_buffer_t *rb, void *item);

// returns the current size of the ring buffer. 
unsigned int ring_buffer_size(ring_buffer_t *rb);

#endif
