#ifndef _SHMAC_PTHREAD_EVENT_LOG_H_
#define _SHMAC_PTHREAD_EVENT_LOG_H_

#include <stddef.h>
#include <stdarg.h>

enum log_event {

	/* the life span of a task */
	TASK_CREATED = 0,
	TASK_ASSIGNED,
	TASK_EXECUTING,
	TASK_STARTED,
	TASK_CONTINUED,
	TASK_STOPPED,
	TASK_CANCELED,
	TASK_EXITED,
	TASK_FINISHED,

	/* joining */
	TASK_JOINING,
	TASK_JOINED,
	TASK_JOIN_BLOCKED,
	TASK_JOIN_VIVIFIED,

	/* irq */
	IRQ_STARTED,
	IRQ_FAILED,
	IRQ_FAILED_CORE_LOCK,
	IRQ_FAILED_MLOCK,
	IRQ_FINISHED,

	/* message queue-related */
	MQ_WORK_REQUEST_SENT,
	MQ_WORK_REQUEST_RECEIVED,
	MQ_WORK_RESPONSE_SENT,
	MQ_WORK_RESPONSE_RECEIVED,

	/* global tq */
	GTQ_INIT,
	GTQ_INSERT,
	GTQ_REMOVE,
	GTQ_LOOKUP_OK,
	GTQ_LOOKUP_FAILED
};

#define logf(event, argc, argv...)			  log_event(*TILEREG_CPUID, event, NULL, argc, ##argv)
#define log_taskf(event, task, argc, argv...) log_event(*TILEREG_CPUID, event, task, argc, ##argv)

#define log(event, ...)			logf(event, 0)
#define log_task(event, task)	log_taskf(event, task, 0)

void log_enable(unsigned int core);
void log_disable(unsigned int core);

void dump_log();
void dump_log_mem(char **location, size_t *bytes_written);

#ifdef ENABLE_EVENT_LOGGING
#include "sched/sched.h"

void log_event(unsigned int core, enum log_event ev, task_t *t, int va_argc, ...);

#else

#define log_event(...)

#endif
#endif
