#include <stdlib.h>
#include <pthread.h>
#include <errno.h>

#include "../sched/sched.h"


static struct sched_param default_schedparam = {
    .sched_priority = 1
};

int pthread_attr_init(pthread_attr_t *attr)
{
    if (attr == NULL) {
        errno = EINVAL;
        return EINVAL;
    }

    attr->stacksize = PTHREAD_STACK_MIN;

    // note: this is not the top of the stack! set
    // fp/sp appropriately elsewhere
    attr->stackaddr = malloc(attr->stacksize + 3);

    if (attr->stackaddr == NULL) {
        errno = ENOMEM;
        return ENOMEM;
    }

    attr->contentionscope = PTHREAD_SCOPE_SYSTEM;
    attr->inheritsched = PTHREAD_EXPLICIT_SCHED;
    attr->schedpolicy = SCHED_OTHER;
    attr->schedparam = default_schedparam;
    attr->detachstate = PTHREAD_CREATE_JOINABLE;

    attr->is_initialized = 1;

    return 0;
}

int pthread_attr_destroy(pthread_attr_t *attr)
{
    attr->is_initialized = 0;
    return 0;
}
