#include <stdlib.h>
#include <pthread.h>
#include <assert.h>

#include "../sched/sched.h"
#include "../core_tq.h"


struct cleanup_item {
    void (*routine)(void*);
    void    *arg;
};


void pthread_cleanup_push(void (*routine)(void*), void *arg)
{
	pthread_t me = pthread_self();

	if (me != PTHREAD_SELF_MASTER) {
		task_t *task = core_tq_get_current();
		assert(task != NULL);

		struct cleanup_item *item = malloc(sizeof(struct cleanup_item));

		if (item != NULL) {
			item->routine = routine;
			item->arg = arg;

			linked_list_push(&(task->cleanup_handlers), item);
		}
	}
}

void pthread_cleanup_pop(int execute)
{
	pthread_t me = pthread_self();

	if (me != PTHREAD_SELF_MASTER) {
		task_t *task = core_tq_get_current();
		assert(task != NULL);

		linkedlist_t *l = &(task->cleanup_handlers);

		if (linked_list_size(l) > 0) {
			linkedlist_item_t *tail = linked_list_tail(l);

			struct cleanup_item *item = tail->value;
			linked_list_remove(l, tail);

			if (execute) {
				item->routine(item->arg);
			}

			free(item);
			free(tail);
		}
	}
}
