#include <pthread.h>
#include <stdlib.h>
#include <errno.h>

#include "../data/rb_tree.h"
#include "pthread_cond.h"

static rb_red_blk_tree *tree = NULL;
static int comp(const void *a, const void *b)
{
	return a < b ? (-1) : (a > b ? 1 : 0);
}

static void key_dest_func(void *_) {}
static void value_dest_func(void *_)  {}
static void print_func(const void *_) {}
static void print_info(void *_) {}
static int pthread_cond_tree_init()
{
	if (tree == NULL) {
		tree = RBTreeCreate(
			comp,
			key_dest_func,
			value_dest_func,
			print_func,
			print_info);

		return tree != NULL;
	}

	return 1;
}

static int pthread_cond_internal_insert(pthread_cond_t *cond,
	const pthread_condattr_t *attr)
{
	if (!pthread_cond_tree_init()) {
		return 0;
	}

	pthread_cond_internal_t *copy = (pthread_cond_internal_t*)malloc(
		sizeof(pthread_cond_internal_t));

	if (copy == NULL) {
		return 0;
	}

	copy->cond = cond;
	if (attr != NULL) {
		copy->attrs = *attr;
	} else {
		pthread_condattr_init(&copy->attrs);
	}
	copy->mutex = NULL;

	linked_list_init(&(copy->listeners));

	void *new_node = RBTreeInsert(tree, (void*)(copy->cond), (void*)copy);
	return new_node != NULL;
}

pthread_cond_internal_t *pthread_cond_internal_find(pthread_cond_t *cond)
{
	if (cond == NULL) {
		return NULL;
	}

	if (!pthread_cond_tree_init()) {
		return NULL;
	}

	rb_red_blk_node *node = RBExactQuery(tree, (void*)cond);
	return node == NULL ? NULL : (pthread_cond_internal_t*)node->info;
}

int pthread_cond_init(pthread_cond_t *cond,
    const pthread_condattr_t *attr)
{

	if (pthread_cond_internal_insert(cond, attr)) {
		*cond = 0;
		return 0;
	}

	return ENOMEM; // probably
}

int pthread_cond_destroy(pthread_cond_t *cond)
{
	*cond = PTHREAD_COND_INITIALIZER;
	return 0;
}
