#include <pthread.h>
#include <errno.h>

#include "../sched/sched.h"
#include "../global_tq.h"
#include "../core.h"
#include "../core_tq.h"

#include "pthread_cond.h"


int pthread_cond_signal(pthread_cond_t *cond)
{
	if (cond == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	pthread_cond_internal_t *internal = pthread_cond_internal_find(cond);
	if (internal == NULL) {
		errno = EINVAL;
		return EINVAL;
	}


	linkedlist_item_t *head = linked_list_pop(&(internal->listeners));
	if (head != NULL) {

		task_t *to_signal = (task_t*)(head->value);
		if (to_signal != NULL) {
			to_signal->cond_waiting = 0;
			core_tq_update_blocked_task(LOCK, to_signal);
		} else {
			warn("bug");
		}
	}


	return 0;
}

int pthread_cond_broadcast(pthread_cond_t *cond)
{
	if (cond == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	pthread_cond_internal_t *internal = pthread_cond_internal_find(cond);
	if (internal == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	linkedlist_item_t *head = linked_list_pop(&(internal->listeners));

	while (head != NULL) {

		task_t *to_signal = (task_t*)(head->value);
		if (to_signal != NULL) {
			core_lock(to_signal->cpu_no, LOCK);

			to_signal->cond_waiting = 0;
			core_tq_update_blocked_task(NONE, to_signal);

			core_unlock(to_signal->cpu_no, 1);
		} else {
			warn("bug");
		}

		head = linked_list_pop(&(internal->listeners));
	}

	return 0;
}
