#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <sched.h>
#include <errno.h>

#include "shmac/shmac.h"
#include "shmac/swi.h"

#include "../sched/sched.h"
#include "../core.h"
#include "../core_tq.h"
#include "../global_tq.h"
#include "../event_log.h"
#include "../task.h"
#include "../boot.h"

static int task_enqueue(sched_entity_t *se)
{
    unsigned int best_core  = UINT32_MAX,
                 best_count = 1000000;

    task_t *task = se->task;

    for (int core = 0; core < *SYS_CPU_COUNT; core++) {
        if (get_core_state(core) == NOT_IN_USE ||
            get_core_state(core) == NOT_READY) {
            continue;
        }

        unsigned int count = core_tq_size(core, task->attrs.schedpolicy);

        if (count < best_count) {
            best_core = core;
            best_count = count;
        }
    }

    if (best_core != UINT32_MAX) {
        log_taskf(TASK_ASSIGNED, task, 2, best_core, best_count);

        global_tq_insert(*(task->thread), task);
        core_tq_push(best_core, task, LOCK);

        debug("task %d was enqueued to %u, who only had %u previous tasks",
            *task->thread, best_core, best_count);

        return 1;
    } else {
        return 0;
    }
}

int pthread_create(pthread_t *t, const pthread_attr_t *attrs,
                   void *(*func)(void *), void *arg)
{
    if (!boot_pthreads_called()) {
        boot_pthreads();
    }

    sched_entity_t *se = task_create(t, attrs, func, arg, 1, 1, 0,
            PTHREAD_CANCEL_ENABLE);

    if (se == NULL) {
        errno = EAGAIN;
        return EAGAIN;
    }

    log_task(TASK_CREATED, se->task);
    if (se->task != NULL && task_enqueue(se)) {
        return 0;
    }

    return EAGAIN;
}
