#include <pthread.h>

#include "shmac/mutex.h"

int pthread_once(pthread_once_t *once_control,
    void (*init_routine)(void))
{
	static shmac_mutex_t mutex = SHMAC_MUTEX_INIT_VALUE;

	if (!once_control->is_initialized) {
		return 0;
	}

	int run = 0;
	lock_mutex(&mutex);

	if (!(once_control->init_executed)) {
		once_control->init_executed = 1;
		run = 1;
	}

	unlock_mutex(&mutex);

	// run outside of locked region to reduce wait-time for others
	if (run) {
		init_routine();
	}

	return 0;
}
