#include <pthread.h>
#include <stdlib.h>
#include <errno.h>

#include "../debug.h"
#include "../data/rb_tree.h"
#include "pthread_rwlock.h"

/*
 * Internals
 */

static rb_red_blk_tree *tree = NULL;
static int comp(const void *a, const void *b)
{
	return a < b ? (-1) : (a > b ? 1 : 0);
}

static void key_dest_func(void *_) {}
static void value_dest_func(void *_)  {}
static void print_func(const void *_) {}
static void print_info(void *_) {}
static int pthread_rwlock_tree_init()
{
	if (tree == NULL) {
		tree = RBTreeCreate(
			comp,
			key_dest_func,
			value_dest_func,
			print_func,
			print_info);

		return tree != NULL;
	}

	return 1;
}


pthread_rwlock_internal_t *pthread_rwlock_internal_find(pthread_rwlock_t *rwlock)
{
	if (!pthread_rwlock_tree_init()) {
		return NULL;
	}

	rb_red_blk_node *node = RBExactQuery(tree, (void*)rwlock);
	return node == NULL ? NULL : (pthread_rwlock_internal_t*)node->info;
}

static int pthread_rwlock_internal_insert(pthread_rwlock_internal_t *rwlock)
{
	if (!pthread_rwlock_tree_init()) {
		return 0;
	}

	RBTreeInsert(tree, (void*)(rwlock->rwlock), (void*)rwlock);

	return 1;
}

static int pthread_rwlock_internal_remove(pthread_rwlock_t *rwlock)
{
	if (!pthread_rwlock_tree_init()) {
		return 0;
	}

	rb_red_blk_node *node = RBExactQuery(tree, (void*)rwlock);
	if (node != NULL) {
		RBDelete(tree, node);
		return 1;
	}

	return 0;
}


int pthread_rwlock_init(pthread_rwlock_t *rwlock,
    const pthread_rwlockattr_t *attr)
{
	if (!pthread_rwlock_tree_init()) {
		return 1;
	}

	pthread_rwlockattr_t att;
	if (attr != NULL) {
		att = *attr;
	} else {
		pthread_rwlockattr_init(&att);
	}

	*rwlock = PTHREAD_RWLOCK_INITIALIZER;

	pthread_rwlock_internal_t *internal = (pthread_rwlock_internal_t*)malloc(
		sizeof(pthread_rwlock_internal_t));
	if (internal == NULL) {
		errno = ENOMEM;
		return ENOMEM;
	}

	internal->rwlock = rwlock;
	internal->attrs = att;

	internal->reads_acquired = 0;
	internal->reads_released = 0;

	internal->writer = PTHREAD_RWLOCK_OWNER_NOBODY;

	internal->lock_obj = PTHREAD_MUTEX_INITIALIZER;
	internal->lock_cond = PTHREAD_COND_INITIALIZER;

	if (pthread_rwlock_internal_insert(internal)) {
		return 0;
	} else {
		return 1;
	}
}
int pthread_rwlock_destroy(pthread_rwlock_t *rwlock)
{
	if (!pthread_rwlock_tree_init()) {
		return 1;
	}

	if (pthread_rwlock_internal_remove(rwlock) != 0) {
		errno = EINVAL;
		return EINVAL;
	}


	return 0;
}
