#include <pthread.h>
#include <errno.h>

#include "../debug.h"
#include "pthread_rwlock.h"

int pthread_rwlock_rdlock(pthread_rwlock_t *rwlock)
{
	if (rwlock == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	pthread_rwlock_internal_t *internal = pthread_rwlock_internal_find(rwlock);
	if (internal == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	if (internal->writer == pthread_self()) {
		errno = EDEADLK;
		return EDEADLK;
	}

	if (pthread_mutex_lock(&(internal->lock_obj)) != 0) {
		return 1;
	}
	while (internal->writer != PTHREAD_RWLOCK_OWNER_NOBODY) {
		if (pthread_cond_wait(&(internal->lock_cond), &(internal->lock_obj)) != 0) {
			return 1;
		}
	}

	internal->reads_acquired++;

	if (pthread_mutex_unlock(&(internal->lock_obj)) != 0) {
		return 1;
	}

	return 0;
}
int pthread_rwlock_tryrdlock(pthread_rwlock_t *rwlock)
{
	if (rwlock == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	pthread_rwlock_internal_t *internal = pthread_rwlock_internal_find(rwlock);
	if (internal == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	if (internal->writer == pthread_self()) {
		errno = EDEADLK;
		return EDEADLK;
	}

	pthread_mutex_lock(&(internal->lock_obj));

	if (internal->writer == PTHREAD_RWLOCK_OWNER_NOBODY) {
		internal->reads_acquired++;
		pthread_mutex_unlock(&(internal->lock_obj));
		return 0;
	} else {
		errno = EBUSY;
		pthread_mutex_unlock(&(internal->lock_obj));
		return EBUSY;
	}

}
