#include <pthread.h>
#include <errno.h>
#include <string.h>

#include "../debug.h"
#include "pthread_rwlock.h"

int pthread_rwlock_wrlock(pthread_rwlock_t *rwlock)
{
	if (rwlock == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	pthread_rwlock_internal_t *internal = pthread_rwlock_internal_find(rwlock);
	if (internal == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	// TODO: detect if a rlock is already held by calling thread
	if (internal->writer == pthread_self()) {
		errno = EDEADLK;
		return EDEADLK;
	}

	if (pthread_mutex_lock(&(internal->lock_obj)) != 0) {
		return 1;
	}
	while (internal->writer != PTHREAD_RWLOCK_OWNER_NOBODY) {
		if (pthread_cond_wait(&(internal->lock_cond), &(internal->lock_obj)) != 0) {
			warn("!!!!! %d = %s", errno, strerror(errno));
		}
	}

	internal->writer = pthread_self();
	
	while (internal->reads_acquired != internal->reads_released) {
		if (pthread_cond_wait(&(internal->lock_cond), &(internal->lock_obj)) != 0) {
			warn("!!! %d = %s", errno, strerror(errno));
		}
	}

	if (pthread_mutex_unlock(&(internal->lock_obj)) != 0) {
		return 1;
	}

	return 0;
}

int pthread_rwlock_trywrlock(pthread_rwlock_t *rwlock)
{
	if (rwlock == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	pthread_rwlock_internal_t *internal = pthread_rwlock_internal_find(rwlock);
	if (internal == NULL) {
		errno = EINVAL;
		return EINVAL;
	}

	// TODO: detect if a rlock is already held by calling thread
	if (internal->writer == pthread_self()) {
		errno = EDEADLK;
		return EDEADLK;
	}

	pthread_mutex_lock(&(internal->lock_obj));
	if (internal->writer == PTHREAD_RWLOCK_OWNER_NOBODY &&
		internal->reads_acquired == internal->reads_released)
	{
		internal->writer = pthread_self();	
		pthread_mutex_unlock(&(internal->lock_obj));
		return 0;
	}
	else
	{
		errno = EBUSY;
		pthread_mutex_lock(&(internal->lock_obj));
		return EBUSY;
	}
}
