#include <pthread.h>
#include <errno.h>

#include "../sched/sched.h"
#include "../global_tq.h"
#include "../core_tq.h"

int pthread_getschedparam(pthread_t thread, int *policy,
    struct sched_param *param)
{
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
	task_t *t = global_tq_find(thread);
	if (t == NULL) {
		errno = ESRCH;
		return ESRCH;
	}

	*policy = t->attrs.schedpolicy;
	*param = t->attrs.schedparam;

	return 0;
#else
	errno = ENOSYS;
	return ENOSYS;
#endif
}

int pthread_setschedparam(pthread_t thread, int policy,
    struct sched_param *param)
{
#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
	if ((
			policy != SCHED_OTHER &&
			policy != SCHED_RR &&
			policy != SCHED_FIFO
		) || (
			param->sched_priority < 1 ||
			param->sched_priority > 100
		))
	{
		errno = EINVAL;
		return EINVAL;
	}

	task_t *t = global_tq_find(thread);
	if (t == NULL) {
		errno = ESRCH;
		return ESRCH;
	}

	int old_policy = t->attrs.schedpolicy;

	t->attrs.schedpolicy = policy;
	t->attrs.schedparam = *param;

	core_tq_update_running_task(LOCK, t, old_policy);
	return 0;
#else
	errno = ENOSYS;
	return ENOSYS;
#endif
}
