#if defined(_POSIX_SPIN_LOCKS)

#include <pthread.h>

/*
 * Since the current mutex implementation that exists in shmac/
 * does not rely on any scheduler and _is_ in fact a spinlock,
 * we just use those here indirectly through the existing mutexes.
 * 
 * The better solution would probably be to copy the implementation
 * of pthread_mutex over here, and rewrite pthread_mutex to better
 * cooperate with the scheduler. But this works, too.
 *
 */

int pthread_spin_init(
    pthread_spinlock_t *__spinlock, int __pshared)
{
    return pthread_mutex_init(__spinlock, NULL);
}
int pthread_spin_destroy(pthread_spinlock_t *__spinlock)
{
    return pthread_mutex_destroy(__spinlock);
}

int pthread_spin_lock(pthread_spinlock_t *__spinlock)
{
    return pthread_mutex_lock(__spinlock);
}

int pthread_spin_trylock(pthread_spinlock_t *__spinlock)
{
    return pthread_mutex_trylock(__spinlock);
}

int pthread_spin_unlock(pthread_spinlock_t *__spinlock)
{
    return pthread_mutex_unlock(__spinlock);
}

#endif /* defined(_POSIX_SPIN_LOCKS) */

