#ifndef _SHMAC_PTHREADS_GLOBAL_TASKS_H_
#define _SHMAC_PTHREADS_GLOBAL_TASKS_H_

#include <pthread.h>
#include <limits.h>

#include "sched/sched.h"

/* Keeps track of tasks globally in a red-black tree. This is
 * used to quickly locate a task when that is needed, for instance
 * in pthread_join. The tasks (pointers, actually) are sorted by
 * pthread_t. That is, tasks are kept in the order they were created.
 */

// Initializes the r-b tree.
void global_tq_init();

// insert a task into the global task tree
void global_tq_insert(pthread_t key, task_t *value);

// remove a task from the global task tree
void global_tq_remove(pthread_t key);

// clean out entire global task queue
void global_tq_reset();

// search for a given task
task_t *global_tq_find(pthread_t key);

// returns the ID of the last thread that was created.
pthread_t global_tq_last_created();

unsigned int global_tq_nof_tasks();

#endif
