#include <stdlib.h>
#include "shmac/shmac.h"

#include "data/linked_list.h"
#include "sched/sched.h"
#include "join_tq.h"

static linkedlist_t joins[MAX_CPUS];

void join_block_task(task_t *task)
{
	task->join_ll_item.value = task;
	linked_list_push_item(&joins[*TILEREG_CPUID], &task->join_ll_item);
}

task_t *join_unblock_task()
{
	linkedlist_item_t *head = linked_list_head(&joins[*TILEREG_CPUID]);

	while (head != NULL) {

		task_t *joiner = (task_t*)head->value;

		if (joiner->waits_for->state == DEQUEUED) {
			linked_list_remove(&joins[*TILEREG_CPUID], head);
			return joiner;
		}

		head = head->next;
	}

	return NULL;
}

int join_blocked_any()
{
	return linked_list_size(&joins[*TILEREG_CPUID]) > 0;
}

#if defined(DEBUG) || defined(WARNINGS)
void join_print(int core)
{
	printf("%d items. ", linked_list_size(&joins[core]));
	linkedlist_item_t *head = linked_list_head(&joins[core]);
	while (head != NULL) {
		task_t *jt = (task_t*)head->value;

		printf("(%d waits for %d st %d)%s",
			*jt->thread,
			*jt->waits_for->thread,
			 jt->waits_for->state,
			 head->next == NULL ? "" : ", "
		);

		head = head->next;
	}
}
#endif
