#ifndef SHMAC_PTHREAD_ENUMS_H_
#define SHMAC_PTHREAD_ENUMS_H_

// The possible states of a schedulable task
enum task_state {
    QUEUED          = 1, // task is created but not yet assigned. may or may not be in global_tq
    RUNNING         = 2, // has been pushed to the runqueue of a CPU
    BLOCKED         = 3, // task has called pthread_cond_{,timed}wait
    FINISHED        = 4, // has finished running, task still in queue, not joined
    DEQUEUED        = 5, // has finished running, task removed from queue, not joined
    REAPED          = 6, // has finished running, either joined or detached
    CANCEL_PENDING  = 7, // har a pending cancel request
    CANCELED        = 8, // has been cancelled completely
};

// the possible states a core can be in
enum core_state {
    NOT_READY   = 0, // hasn't called core_init yet
    NOT_IN_USE  = 1, // master core, for instance
    IDLE        = 2, // a core is idle until it has been given a task for the first time.
    BUSY        = 3  // busy working (even if that is the idle task)
};

// used to specify the locking semantics of certain functions in core_tq.
enum lock_mode {
    LOCK     = 0,
    TRYLOCK  = 1,
    NONE     = 2
};


// used internally to specify why a certain task should block. this decides
// which blocking queue it will be inserted into.
enum task_block_reason {
    BLOCK_COND          = 0,
    BLOCK_COND_TIME     = 1,
    BLOCK_BARRIER       = 2,

    BLOCK_REASON_MIN    = BLOCK_COND,
    BLOCK_REASON_MAX    = BLOCK_BARRIER
};

// used to send messages between cores. used for work requesting/responses
enum core_msg {
    WORK_REQUEST  = 0,
    WORK_RESPONSE = 1
};

#endif
