#ifndef SHMAC_PTHREADS_SCHED_HEAP_H_
#define SHMAC_PTHREADS_SCHED_HEAP_H_

#include "sched.h"

struct sched_entity {
    struct task_struct      *task;

    unsigned int             epoch_sched_count;

    unsigned int             vruntime;      // virtual runtime, in system ticks
    unsigned int             created_time;  // when the task was created
    unsigned int             start;         // when this task was first started
    unsigned int             last_start;    // last time we started running (in ticks)
};

void sched_heap_init();
int sched_heap_inited();

void sched_heap_task_start_hook(task_t *task);
void sched_heap_task_retire_hook(task_t *task);
void sched_heap_task_continue_hook(task_t *task);
void sched_heap_task_tick_hook(task_t *task);

void sched_heap_entity_init(struct sched_entity *se, struct task_struct *task);
void sched_heap_task_pre_push_hook(core_t *core, task_t *new_task);

#define sched_init(...) sched_heap_init(__VA_ARGS__)
#define sched_inited(...) sched_heap_inited(__VA_ARGS__)

#define sched_task_start_hook(...) sched_heap_task_start_hook(__VA_ARGS__)
#define sched_task_retire_hook(...) sched_heap_task_retire_hook(__VA_ARGS__)
#define sched_task_continue_hook(...) sched_heap_task_continue_hook(__VA_ARGS__)
#define sched_task_tick_hook(...) sched_heap_task_tick_hook(__VA_ARGS__)

#define sched_entity_init(...) sched_heap_entity_init(__VA_ARGS__)
#define sched_task_pre_push_hook(...) sched_heap_task_pre_push_hook(__VA_ARGS__)

#define prio_array_init(...) heap_init(__VA_ARGS__)
#define prio_array_peek(...) heap_peek(__VA_ARGS__)
#define prio_array_push(...) heap_push(__VA_ARGS__)
#define prio_array_pop(...) heap_pop(__VA_ARGS__)
#define prio_array_pop_push(...) heap_pop_push(__VA_ARGS__)
#define prio_array_beats_root(...) heap_beats_root(__VA_ARGS__)
#define prio_array_pop_first(...) heap_pop_first(__VA_ARGS__)
#define prio_array_update_first(...) heap_update_first(__VA_ARGS__)

#define prio_array_capacity(...) heap_capacity(__VA_ARGS__)
#define prio_array_size(...) heap_size(__VA_ARGS__)

#if defined(DEBUG) || defined(WARNINGS)
#define prio_array_print(...) heap_print(__VA_ARGS__)
#endif

#endif
