#ifndef SHMAC_PTHREAD_TASK_TYPE_H_
#define SHMAC_PTHREAD_TASK_TYPE_H_

#include <pthread.h>

#include "shmac/irq.h"

#include "enums.h"
#include "../data/linked_list.h"

// The type of functions pthread_create expects passed to it.
typedef void *(*func_t)(void* arg);

struct sched_entity;

struct task_struct {
    volatile enum task_state state;             // state, set by both master and slave
    volatile int             cpu_no;            // which core are we executing on?

    pthread_t               *thread;            // the unique pthread_t for this task
    pthread_attr_t           attrs;             // thread attributes

    int                      stack_free;        // should we free the stack pointer?
    int                      task_free;         // should we free the task_t* itself?

    func_t                   task;              // the user-specified function to execute
    void                    *arg;               // argument to pass to task
    void                    *ret;               // return value from task

    struct task_struct      *waits_for;         // used to detect deadlocks when joining
    linkedlist_item_t        join_ll_item;

    int                      pinned;            // can this task be moved to another core?
    int                      blockable;         // used to prevent blocking when holding locks

    // blocking flags
    volatile int             cond_waiting;      // waiting on a condition variable. will not be scheduled.
    volatile unsigned int    cond_wait_timeout; // the *SYS_TICKS value at which the thread unblocks
    volatile int             barrier_waiting;   // waiting at a barrier
    volatile int             join_waiting;      // is waiting on another thread

    int                      cancel_state;      // ENABLED / DISABLED
    int                      cancel_type;       // DEFERRED / ASYNC. we do not support DEFERRED

    irq_regs_t               register_set;      // the state to go back to later

    linkedlist_t             cleanup_handlers;  // functions to be called upon cancelation


    // count the number of times this task has been preempted
    unsigned int             epoch_count;

    struct sched_entity     *se;
};

typedef struct task_struct task_t;
typedef struct sched_entity sched_entity_t;

#endif
