#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <assert.h>
#include <pthread.h>

#include "../global_tq.h"

#include "test.h"
#include "pthreads_emb.h"

void sched_yield(void);

enum
{
  NUMTHREADS = 10
};

static void test_init()
{

}

int test_pthreads_embedded_all()
{
	//test_pthreads_embedded_create1();
	//test_pthreads_embedded_create2();
	//test_pthreads_embedded_create3();

	//test_pthreads_embedded_exit1();
	//test_pthreads_embedded_exit2();
	//test_pthreads_embedded_exit3();
	//test_pthreads_embedded_exit4();
	//test_pthreads_embedded_exit5();

	//test_pthreads_embedded_join1();
	//test_pthreads_embedded_join2();
	//test_pthreads_embedded_join3();

	test_pthreads_embedded_mutex_normal();
	test_pthreads_embedded_mutex_errorcheck();
	test_pthreads_embedded_mutex_recursive();
	test_pthreads_embedded_mutex_2();
	test_pthreads_embedded_mutex_3();
	test_pthreads_embedded_mutex_4();

	return 1;
}

void test_pthreads_embedded_create1()
{
	test_init();
	print_function_name();

	static volatile int washere = 0;
	void *func(void * arg)
	{
		//  washere = 1;
		washere++;
		return (void *) 0;
	}
	pthread_t t;

	washere = 0;

	assert(pthread_create(&t, NULL, func, NULL) == 0);

	/* A dirty hack, but we cannot rely on pthread_join in this
	   primitive test. */
	sleep(500);

	assert(washere == 1);
	assert(pthread_join(t,NULL) == 0);
}

void test_pthreads_embedded_create2()
{
	test_init();
	print_function_name();

	static volatile int washere = 0;
	void *func(void * arg)
	{
		//  washere = 1;
		washere++;
		return (void *) 0;
	}

	pthread_t t;
	pthread_attr_t attr;
	void * result = NULL;
	int i;

	washere = 0;

	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);

	for (i = 0; i < NUMTHREADS; i++)
	{
		washere = 0;
		assert(pthread_create(&t, NULL, func, NULL) == 0);
		pthread_join(t, &result);
		assert(washere == 1);
	}
}

void test_pthreads_embedded_create3()
{
	test_init();
	print_function_name();

	void *threadFunc(void * arg)
	{
		return (void *) 0;
	}

	int i;
	pthread_t mt;
	pthread_t t[NUMTHREADS];

	assert((mt = pthread_self()) != 0);

	/* Avoid compiler warning */
	mt = mt;

	for (i = 0; i < NUMTHREADS; i++)
	{
		assert(pthread_create(&t[i], NULL, &threadFunc, NULL) == 0);
	}

	for (i = 0; i < NUMTHREADS; i++)
	{
		assert(pthread_join(t[i],NULL) == 0);
	}
}

void test_pthreads_embedded_exit1()
{
	test_init();
	print_function_name();

	void *func(void * param)
	{
	  pthread_exit((void *) 0);

	  /* Not reached */
	  assert(0);
	}

	pthread_t id;
	/* A simple test first. */

	assert(pthread_create(&id, NULL, func, 0) == 0);

	// Don't want to depend on join.
	sleep(500);

	assert(pthread_join(id,NULL) == 0);
}

void test_pthreads_embedded_exit2()
{
	test_init();
	print_function_name();

	void *func(void * arg)
	{
		pthread_exit(arg);

		/* Never reached. */
		assert(0);

		return NULL;
	}

	pthread_t t;

	assert(pthread_create(&t, NULL, func, (void *) NULL) == 0);

	sleep(500);

	assert(pthread_join(t,NULL) == 0);
}

void test_pthreads_embedded_exit3()
{
	test_init();
	print_function_name();
	void *func(void * arg)
	{
		pthread_exit(arg);

		/* Never reached. */
		assert(0);

		return NULL;
	}

	pthread_t id[4];
	int i;

	/* Create a few threads and then exit. */
	for (i = 0; i < 4; i++)
	{
		assert(pthread_create(&id[i], NULL, func, (void *) i) == 0);
	}

	sleep(1000);

	for (i = 0; i < 4; i++)
	{
		void * retValue;
		assert(pthread_join(id[i],&retValue) == 0);
		assert((int)retValue == i);
	}
}

void test_pthreads_embedded_exit4()
{
	test_init();
	print_function_name();

}

void test_pthreads_embedded_exit5()
{
	test_init();
	print_function_name();

}


void test_pthreads_embedded_join1()
{
	test_init();
	print_function_name();

	void *func(void * arg)
	{
		sleep(2000);

		pthread_exit(arg);

		/* Never reached. */
		exit(1);

		return 0;
	}

	pthread_t id;
	int result;

	/* Create a single thread and wait for it to exit. */
	assert(pthread_create(&id, NULL, func, (void *) 123) == 0);

	assert(pthread_join(id, (void **) &result) == 0);

	assert(result == 123);
}

void test_pthreads_embedded_join2()
{
	test_init();
	print_function_name();

	void *func(void * arg)
	{
		sleep(2000);
		return arg;
	}

	pthread_t id[4];
	int i;
	int result;

	/* Create a few threads and then exit. */
	for (i = 0; i < 4; i++)
	{
		assert(pthread_create(&id[i], NULL, func, (void *) i) == 0);
	}

	for (i = 0; i < 4; i++)
	{
		assert(pthread_join(id[i], (void **) &result) == 0);
		assert(result == i);
	}
}

void test_pthreads_embedded_join3()
{
	test_init();
	print_function_name();

	void *func(void * arg)
	{
		sched_yield();
		return arg;
	}

	pthread_t id[4];
	int i;
	int result;

	/* Create a few threads and then exit. */
	for (i = 0; i < 4; i++)
	{
		assert(pthread_create(&id[i], NULL, func, (void *) i) == 0);
	}

	/*
	 * Let threads exit before we join them.
	 * We should still retrieve the exit code for the threads.
	 */
	sleep(1000);

	for (i = 0; i < 4; i++)
	{
		assert(pthread_join(id[i], (void **) &result) == 0);
		assert(result == i);
	}
}

void test_pthreads_embedded_mutex_normal()
{
	test_init();
	print_function_name();

	static pthread_mutex_t mutex = NULL;

	assert(pthread_mutex_init(&mutex, NULL) == 0);

	assert(pthread_mutex_lock(&mutex) == 0);
	assert(pthread_mutex_unlock(&mutex) == 0);
	assert(pthread_mutex_destroy(&mutex) == 0);
}

void test_pthreads_embedded_mutex_errorcheck()
{
	test_init();
	print_function_name();

	static pthread_mutex_t mutex = NULL;
	static pthread_mutexattr_t mxAttr;

	assert(pthread_mutexattr_init(&mxAttr) == 0);
	assert(pthread_mutexattr_settype(&mxAttr, PTHREAD_MUTEX_ERRORCHECK) == 0);
	assert(pthread_mutex_init(&mutex, &mxAttr) == 0);
	assert(pthread_mutex_lock(&mutex) == 0);
	assert(pthread_mutex_unlock(&mutex) == 0);
	assert(pthread_mutex_destroy(&mutex) == 0);
}

void test_pthreads_embedded_mutex_recursive()
{
	test_init();
	print_function_name();

	static pthread_mutex_t mutex = NULL;
	static pthread_mutexattr_t mxAttr;

	assert(pthread_mutexattr_init(&mxAttr) == 0);
	assert(pthread_mutexattr_settype(&mxAttr, PTHREAD_MUTEX_RECURSIVE) == 0);
	assert(pthread_mutex_init(&mutex, &mxAttr) == 0);
	assert(pthread_mutex_lock(&mutex) == 0);
	assert(pthread_mutex_unlock(&mutex) == 0);
	assert(pthread_mutex_destroy(&mutex) == 0);
}

void test_pthreads_embedded_mutex_2()
{
	test_init();
	print_function_name();

	static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
	static int washere = 0;

	void * func(void * arg)
	{
		assert(pthread_mutex_trylock(&mutex1) == EBUSY);
		washere = 1;
		return (void*)0;
	}

	pthread_t t;
	mutex1 = PTHREAD_MUTEX_INITIALIZER;

	assert(pthread_mutex_lock(&mutex1) == 0);

	assert(pthread_create(&t, NULL, func, NULL) == 0);
	assert(pthread_join(t, NULL) == 0);

	assert(pthread_mutex_unlock(&mutex1) == 0);
	assert(washere == 1);
	assert(pthread_mutex_destroy(&mutex1) == 0);
}

void test_pthreads_embedded_mutex_3()
{
	test_init();
	print_function_name();

	static int wasHere = 0;
	static pthread_mutex_t mutex1;

	void * unlocker(void * arg)
	{
		int expectedResult = (int) arg;

		wasHere++;
		assert(pthread_mutex_unlock(&mutex1) == expectedResult);
		wasHere++;
		return NULL;
	}

	pthread_t t;
	pthread_mutexattr_t ma;

	assert(pthread_mutexattr_init(&ma) == 0);

	wasHere = 0;
	assert(pthread_mutexattr_settype(&ma, PTHREAD_MUTEX_DEFAULT) == 0);
	assert(pthread_mutex_init(&mutex1, &ma) == 0);
	assert(pthread_mutex_lock(&mutex1) == 0);
	/*
	 * NORMAL (fast) mutexes don't check ownership.
	 */
	assert(pthread_create(&t, NULL, unlocker, (void *) 0) == 0);
	assert(pthread_join(t, NULL) == 0);
	assert(pthread_mutex_unlock(&mutex1) == 0);
	assert(wasHere == 2);
	assert(pthread_mutex_destroy(&mutex1) == 0);

	wasHere = 0;
	assert(pthread_mutexattr_settype(&ma, PTHREAD_MUTEX_NORMAL) == 0);
	assert(pthread_mutex_init(&mutex1, &ma) == 0);
	assert(pthread_mutex_lock(&mutex1) == 0);
	/*
	 * NORMAL (fast) mutexes don't check ownership.
	 */
	assert(pthread_create(&t, NULL, unlocker, (void *) 0) == 0);
	assert(pthread_join(t, NULL) == 0);
	assert(pthread_mutex_unlock(&mutex1) == 0);
	assert(wasHere == 2);
	assert(pthread_mutex_destroy(&mutex1) == 0);


	wasHere = 0;
	assert(pthread_mutexattr_settype(&ma, PTHREAD_MUTEX_ERRORCHECK) == 0);
	assert(pthread_mutex_init(&mutex1, &ma) == 0);
	assert(pthread_mutex_lock(&mutex1) == 0);
	assert(pthread_create(&t, NULL, unlocker, (void *) EPERM) == 0);
	assert(pthread_join(t, NULL) == 0);
	assert(pthread_mutex_unlock(&mutex1) == 0);
	assert(wasHere == 2);
	assert(pthread_mutex_destroy(&mutex1) == 0);

	wasHere = 0;
	assert(pthread_mutexattr_settype(&ma, PTHREAD_MUTEX_RECURSIVE) == 0);
	assert(pthread_mutex_init(&mutex1, &ma) == 0);
	assert(pthread_mutex_lock(&mutex1) == 0);
	assert(pthread_create(&t, NULL, unlocker, (void *) EPERM) == 0);
	assert(pthread_join(t, NULL) == 0);
	assert(pthread_mutex_unlock(&mutex1) == 0);
	assert(wasHere == 2);
	assert(pthread_mutex_destroy(&mutex1) == 0);
}

void test_pthreads_embedded_mutex_4()
{
	test_init();
	print_function_name();

	static int lockCount = 0;

	static pthread_mutex_t mutex;
	static pthread_mutexattr_t mxAttr;

	void * locker(void * arg)
	{
		assert(pthread_mutex_lock(&mutex) == 0);
		lockCount++;
		assert(pthread_mutex_lock(&mutex) == EDEADLK);
		lockCount++;
		assert(pthread_mutex_unlock(&mutex) == 0);
		assert(pthread_mutex_unlock(&mutex) == EPERM);

		return (void *) 555;
	}

	pthread_t t;
	int result = 0;
	int mxType = -1;

	lockCount = 0;
	assert(pthread_mutexattr_init(&mxAttr) == 0);
	assert(pthread_mutexattr_settype(&mxAttr, PTHREAD_MUTEX_ERRORCHECK) == 0);
	assert(pthread_mutexattr_gettype(&mxAttr, &mxType) == 0);
	assert(mxType == PTHREAD_MUTEX_ERRORCHECK);

	assert(pthread_mutex_init(&mutex, &mxAttr) == 0);

	assert(pthread_create(&t, NULL, locker, NULL) == 0);

	assert(pthread_join(t, (void **) &result) == 0);
	assert(result == 555);

	assert(lockCount == 2);

	assert(pthread_mutex_destroy(&mutex) == 0);
	assert(pthread_mutexattr_destroy(&mxAttr) == 0);
}
