#include <stdio.h>
#include <assert.h>
#include <pthread.h>

#include "../global_tq.h"

#include "test.h"
#include "pthreads_threading.h"


static void test_init()
{

}

int test_pthreads_threading_all()
{
	test_pthreads_threading_create();
	test_pthreads_threading_argument();
	test_pthreads_threading_join();

	test_pthreads_threading_exit();
	test_pthreads_threading_cancel();
	test_pthreads_threading_detach();

	return 1;
}


/* tests thread creation. will launch a thread
 * and simply verify after a short sleep that
 * the thread function was actually called.
 */
static int threading_function_called = 0;
static void *test_pthreads_threading_create_func(void *arg)
{
	threading_function_called = 1;
	return NULL;
}
void test_pthreads_threading_create()
{
	test_init();
	print_function_name();

	pthread_t tid;
	assert(pthread_create(&tid, NULL, &test_pthreads_threading_create_func, NULL) == 0);

	sleep(500);

	assert(threading_function_called == 1);
}


/* tests that the specified parameter is passed to
 * the threading function
 */
static void *threading_arg_passed = 0;
static void *test_pthreads_threading_arg_passed(void *arg)
{
	threading_arg_passed = arg;
	return NULL;
}
void test_pthreads_threading_argument()
{
	test_init();
	print_function_name();

	void *arg = (void*)0x42;

	pthread_t tid;
	assert(pthread_create(&tid, NULL, &test_pthreads_threading_arg_passed, arg) == 0);

	sleep(500);

	assert(threading_arg_passed == arg);
}

void test_pthreads_threading_join()
{
	void *foo(void *arg)
	{
		return (void*)42;
	}

	test_init();
	print_function_name();

	pthread_t tid;
	assert(pthread_create(&tid, NULL, &foo, NULL) == 0);

	void *ret;
	assert(pthread_join(tid, &ret) == 0);

	assert((int)ret == 42);
}


void test_pthreads_threading_exit()
{
	void *foo(void *arg)
	{
		pthread_exit((void*)42);
		return NULL;
	}

	test_init();
	print_function_name();

	pthread_t tid;
	assert(pthread_create(&tid, NULL, &foo, NULL) == 0);

	void *ans;
	assert(pthread_join(tid, &ans) == 0);
	assert((int)ans == 42);
}

void test_pthreads_threading_cancel()
{
	volatile int loops = 0;
	void *foo(void *arg)
	{
		while (1) {
			loops++;
		}
		return NULL;
	}

	test_init();
	print_function_name();

	pthread_t tid;
	assert(pthread_create(&tid, NULL, &foo, NULL) == 0);
	assert(pthread_cancel(tid) == 0);

	uint32_t stop = *SYS_TICKS + SYS_TICKS_RATE / 5;
	while (*SYS_TICKS < stop);

	int l = loops;
	assert(l == loops);

	stop = *SYS_TICKS + SYS_TICKS_RATE / 1000;
	while (*SYS_TICKS < stop);

	assert(l == loops);

	// ok, foo has probably stopped at this point..
}

void test_pthreads_threading_detach()
{
	volatile int x = 0;
	void *foo(void *arg)
	{
		uint32_t stop = *SYS_TICKS + SYS_TICKS_RATE / 5;
		while (*SYS_TICKS < stop);
		return (void*)(x = 42);
	}

	test_init();
	print_function_name();

	pthread_t tid;
	assert(pthread_create(&tid, NULL, &foo, NULL) == 0);
	assert(pthread_detach(tid) == 0);

	uint32_t ss = *SYS_TICKS + SYS_TICKS_RATE / 2;
	while (*SYS_TICKS < ss);

	assert(x == 42);

	assert(global_tq_find(tid) == NULL);
}
