#include <unistd.h>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include  "../data/rb_tree.h"

#include "test.h"
#include "rb_tree.h"

static int ds1[] = { 0, 4, 7, 3, 1, 9, 10 }; // [0] has to be smallest


static rb_red_blk_tree *tree = NULL;

static int comp(const void *a, const void *b)
{
	return a < b ? (-1) : (a > b ? 1 : 0);
}

static void key_dest_func(void *key) {}
static void value_dest_func(void *task_ptr) {}
static void print_func(const void *_){}
static void print_info(void *_){}

static void reset_tree()
{
	if (tree != NULL) {
		RBTreeDestroy(tree);
	}

	tree = RBTreeCreate(
		comp,
		key_dest_func,
		value_dest_func,
		print_func,
		print_info);

	assert(tree != NULL);
}

static int inited = 0;
static void rbtree_init()
{
	if (!inited) {
		reset_tree();
		inited = 1;
	}
}

int test_rbtree_all()
{
	rbtree_init();

	test_rbtree_create();

	test_rbtree_insert_search();
	test_rbtree_remove();

	test_rbtree_first();

	return 1;
}

void test_rbtree_create()
{
	print_function_name();

	rbtree_init();
	assert(tree != NULL);
}

void test_rbtree_insert_search()
{
	print_function_name();
	reset_tree();

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBTreeInsert(tree, (void*)ds1[i], (void*)(ds1[i] + 1));
		assert(node != NULL);
	}

	// test that all items are found
	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *result = RBExactQuery(tree, (void*)ds1[i]);
		assert(result != NULL);
	}
}

void test_rbtree_remove()
{
	print_function_name();
	reset_tree();

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBTreeInsert(tree, (void*)ds1[i], (void*)(ds1[i] + 1));
		assert(node != NULL);
	}

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBExactQuery(tree, (void*)ds1[i]);
		assert(node != NULL);
		RBDelete(tree, node);

		node = RBExactQuery(tree, (void*)ds1[i]);
		assert(node == NULL);
	}

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBExactQuery(tree, (void*)ds1[i]);
		assert(node == NULL);
	}
}


void test_rbtree_first()
{
	print_function_name();
	reset_tree();

	rb_red_blk_node *node = TreeFirstNode(tree);
	assert(node != NULL);
	assert(node->parent == tree->nil);
	assert(node->left == tree->nil);
	assert(node->right == tree->nil);

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBTreeInsert(tree, (void*)ds1[i], (void*)(ds1[i] + 1));
		assert(node != NULL);

		node = TreeFirstNode(tree);
		assert(node != NULL);
		assert((int)(node->key) == ds1[0]);
	}

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBExactQuery(tree, (void*)ds1[i]);
		assert(node != NULL);
		RBDelete(tree, node);

		node = RBExactQuery(tree, (void*)ds1[i]);
		assert(node == NULL);
	}

	for (int i = 0; i < sizeof(ds1) / sizeof(ds1[0]); i++)
	{
		rb_red_blk_node *node = RBExactQuery(tree, (void*)ds1[i]);
		assert(node == NULL);
	}
}
