#include <stdio.h>
#include <assert.h>

#include  "../data/ring_buffer.h"

#include "test.h"
#include "ring_buffer.h"


static ring_buffer_t buffer;
static ring_buffer_t *buf = &buffer;
static int buffer_inited = 0;

static void test_ring_buffer_init()
{
	if (!buffer_inited) {
		buf->buffer_size = 4;
		ring_buffer_init(buf);

		buffer_inited = 1;
	}
}
static void test_ring_buffer_reset()
{
	buf->head = 0;
	buf->tail = 0;
}

int test_ring_buffer_all()
{
	test_ring_buffer_init();

	test_ring_buffer_push();
	test_ring_buffer_pop();

	test_ring_buffer_push_wrapped();
	test_ring_buffer_pop_wrapped();

	test_ring_buffer_size();
	test_ring_buffer_size_wrapped();

	return 1;
}

void test_ring_buffer_push()
{
	test_ring_buffer_reset();
	print_function_name();

	assert(buf->head == 0);
	assert(buf->tail == 0);
	ring_buffer_push(buf, (void*)0x42);
	assert(buf->head == 0);
	assert(buf->tail == 1);

	ring_buffer_push(buf, (void*)0x43);
	assert(buf->head == 0);
	assert(buf->tail == 2);

	ring_buffer_push(buf, (void*)0x44);
	assert(buf->head == 0);
	assert(buf->tail == 3);
}

void test_ring_buffer_pop()
{
	test_ring_buffer_reset();
	print_function_name();

	ring_buffer_push(buf, (void*)0x0);
	ring_buffer_push(buf, (void*)0x1);
	ring_buffer_push(buf, (void*)0x2);

	assert(buf->head == 0);
	assert(buf->tail == 3);

	ring_buffer_pop(buf);
	assert(buf->head == 1);
	assert(buf->tail == 3);

	ring_buffer_pop(buf);
	assert(buf->head == 2);
	assert(buf->tail == 3);

	ring_buffer_pop(buf);
	assert(buf->head == 3);
	assert(buf->tail == 3);
}

void test_ring_buffer_push_wrapped()
{
	test_ring_buffer_reset();
	print_function_name();

	ring_buffer_push(buf, (void*)0x0);
	ring_buffer_push(buf, (void*)0x1);
	ring_buffer_push(buf, (void*)0x2);

	ring_buffer_pop(buf);
	ring_buffer_pop(buf);
	ring_buffer_pop(buf);

	// head/tail is now 3/3
	assert(buf->head == 3);
	assert(buf->tail == 3);

	ring_buffer_push(buf, (void*)0x3);

	// head/tail is now 3/0
	assert(buf->head == 3);
	assert(buf->tail == 0);

	ring_buffer_push(buf, (void*)0x4);

	// head/tail is now 3/1
	assert(buf->head == 3);
	assert(buf->tail == 1);

	ring_buffer_push(buf, (void*)0x4);

	// head/tail is now 3/2
	assert(buf->head == 3);
	assert(buf->tail == 2);

}
void test_ring_buffer_pop_wrapped()
{
	test_ring_buffer_reset();
	print_function_name();
}

void test_ring_buffer_size()
{
	test_ring_buffer_reset();
	print_function_name();

	assert(ring_buffer_size(buf) == 0);

	ring_buffer_push(buf, (void*)0x0);
	assert(ring_buffer_size(buf) == 1);

	ring_buffer_push(buf, (void*)0x1);
	assert(ring_buffer_size(buf) == 2);

	ring_buffer_push(buf, (void*)0x2);
	assert(ring_buffer_size(buf) == 3);

	ring_buffer_pop(buf);
	assert(ring_buffer_size(buf) == 2);

	ring_buffer_pop(buf);
	assert(ring_buffer_size(buf) == 1);

	ring_buffer_pop(buf);
	assert(ring_buffer_size(buf) == 0);
}

void test_ring_buffer_size_wrapped()
{
	test_ring_buffer_reset();
	print_function_name();

	ring_buffer_push(buf, (void*)0x0);
	ring_buffer_push(buf, (void*)0x1);

	ring_buffer_pop(buf);
	ring_buffer_pop(buf);

	// head/tail should now be 2/2

	ring_buffer_push(buf, (void*)0x2);

	// head/tail should now be 2/3

	ring_buffer_push(buf, (void*)0x3);

	// head/tail should now be 2/0

	assert(ring_buffer_size(buf) == 2);

	ring_buffer_pop(buf);

	// head/tail should now be 3/0

	assert(ring_buffer_size(buf) == 1);

	ring_buffer_pop(buf);

	// head/tail should now be 0/0

	assert(ring_buffer_size(buf) == 0);
}
